#!/usr/bin/env python3

# Copyright 2014-2015 Open Source Robotics Foundation, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
from collections import OrderedDict
import os
import sys

from catkin_pkg.package import parse_package_string


def main(argv=sys.argv[1:]):
    """
    Extract the information from package.xml and make them accessible to CMake.

    Parse the given package.xml file and
    print CMake code defining several variables containing the content.
    """
    parser = argparse.ArgumentParser(
        description='Parse package.xml file and print CMake code defining '
                    'several variables',
    )
    parser.add_argument(
        'package_xml',
        type=argparse.FileType('r', encoding='utf-8'),
        help='The path to a package.xml file',
    )
    parser.add_argument(
        'outfile',
        nargs='?',
        help='The filename where the output should be written to',
    )
    args = parser.parse_args(argv)

    try:
        package = parse_package_string(
            args.package_xml.read(), filename=args.package_xml.name)
    except Exception as e:
        print("Error parsing '%s':" % args.package_xml.name, file=sys.stderr)
        raise e
    finally:
        args.package_xml.close()

    lines = generate_cmake_code(package)
    if args.outfile:
        with open(args.outfile, 'w', encoding='utf-8') as f:
            for line in lines:
                f.write('%s\n' % line)
    else:
        for line in lines:
            print(line)


def get_dependency_values(key, depends):
    dependencies = []

    # Filter the dependencies, checking for any condition attributes
    dependencies.append((key, ' '.join([
        '"%s"' % str(d) for d in depends
        if d.condition is None or d.evaluate_condition(os.environ)
    ])))

    for d in depends:
        comparisons = [
            'version_lt',
            'version_lte',
            'version_eq',
            'version_gte',
            'version_gt']
        for comp in comparisons:
            value = getattr(d, comp, None)
            if value is not None:
                dependencies.append(('%s_%s_%s' % (key, str(d), comp.upper()),
                                     '"%s"' % value))
    return dependencies


def generate_cmake_code(package):
    """
    Return a list of CMake set() commands containing the manifest information.

    :param package: catkin_pkg.package.Package
    :returns: list of str
    """
    variables = []
    variables.append(('VERSION', '"%s"' % package.version))

    variables.append((
        'MAINTAINER',
        '"%s"' % (', '.join([str(m) for m in package.maintainers]))))

    variables.extend(get_dependency_values('BUILD_DEPENDS',
                                           package.build_depends))
    variables.extend(get_dependency_values('BUILDTOOL_DEPENDS',
                                           package.buildtool_depends))
    variables.extend(get_dependency_values('BUILD_EXPORT_DEPENDS',
                                           package.build_export_depends))
    variables.extend(get_dependency_values('BUILDTOOL_EXPORT_DEPENDS',
                                           package.buildtool_export_depends))
    variables.extend(get_dependency_values('EXEC_DEPENDS',
                                           package.exec_depends))
    variables.extend(get_dependency_values('TEST_DEPENDS',
                                           package.test_depends))
    variables.extend(get_dependency_values('GROUP_DEPENDS',
                                           package.group_depends))
    variables.extend(get_dependency_values('MEMBER_OF_GROUPS',
                                           package.member_of_groups))

    deprecated = [e.content for e in package.exports
                  if e.tagname == 'deprecated']
    variables.append(('DEPRECATED',
                      '"%s"' % ((deprecated[0] if deprecated[0] else 'TRUE')
                                if deprecated
                                else '')))

    lines = []
    lines.append('set(_AMENT_PACKAGE_NAME "%s")' % package.name)
    for (k, v) in variables:
        lines.append('set(%s_%s %s)' % (package.name, k, v))

    lines.append('set(%s_EXPORT_TAGS)' % package.name)
    replaces = OrderedDict()
    replaces['${prefix}/'] = ''
    replaces['\\'] = '\\\\'  # escape backslashes
    replaces['"'] = '\\"'  # prevent double quotes to end the CMake string
    replaces[';'] = '\\;'  # prevent semicolons to be interpreted as list separators
    for export in package.exports:
        export = str(export)
        for k, v in replaces.items():
            export = export.replace(k, v)
        lines.append('list(APPEND %s_EXPORT_TAGS "%s")' % (package.name, export))

    return lines


if __name__ == '__main__':
    main()



# Copyright 2020 Open Source Robotics Foundation, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import json
import os
import subprocess
import sys


common_test_properties = {
    'name',
    'run_name',
    'run_type',
    'repetitions',
    'repetition_index',
    'threads',
    'time_unit',
}


common_aggregate_test_properties = common_test_properties | {
    'aggregate_name',
}


common_iteration_test_properties = common_test_properties | {
    'iterations',
    'real_time',
    'cpu_time',
}


def main(argv=sys.argv[1:]):
    parser = argparse.ArgumentParser(
        description='Run a Google Benchmark test and convert the results to '
                    'a common format.')
    parser.add_argument(
        'result_file_in', help='The path to the Google Benchmark result file')
    parser.add_argument(
        'result_file_out',
        help='The path to where the common result file should be written')
    parser.add_argument(
        '--package-name',
        default=None,
        help="The package name to be used as a prefix for the 'group' "
             'value in benchmark result files')
    parser.add_argument(
        '--command',
        nargs='+',
        help='The test command to execute. '
             'It must be passed after other arguments since it collects all '
             'following options.')
    parser.add_argument(
        '--result-file-overlay',
        default=None,
        help='If specified, this json file will be overlaid on top of the '
             'generated results. This is primarily useful for specifying '
             'descriptions and/or thresholds.')
    if '--command' in argv:
        index = argv.index('--command')
        argv, command = argv[0:index + 1] + ['dummy'], argv[index + 1:]
    args = parser.parse_args(argv)
    args.command = command

    print('Executing benchmark test command: %s\n\n' % ' '.join(args.command))
    res = subprocess.run(args.command)
    print('\n\nTest command returned result status %d' % res.returncode)

    try:
        with open(args.result_file_in, 'r') as in_file:
            in_text = in_file.read()
    except FileNotFoundError:
        if res.returncode == 0:
            print(
                'ERROR: No performance test results were found at: %s' % args.result_file_in,
                file=sys.stderr)
            res.returncode = 1
        return res.returncode

    if not in_text and res.returncode == 0:
        print(
            'NOTE: Performance test results file was empty at: %s' % args.result_file_in,
            file=sys.stderr)
        open(args.result_file_out, 'w').close()
        return res.returncode

    try:
        in_data = json.loads(in_text)
    except json.decoder.JSONDecodeError as e:
        print(
            'Failure parsing performance results file at: %s' % args.result_file_in,
            file=sys.stderr)
        print(e)
        if res.returncode == 0:
            res.returncode = 1
        return res.returncode

    overlay_data = None
    if args.result_file_overlay:
        with open(args.result_file_overlay, 'r') as overlay_file:
            overlay_data = json.load(overlay_file)
    out_data = convert_google_benchark_to_jenkins_benchmark(
        in_data, overlay_data, args.package_name)
    with open(args.result_file_out, 'w') as out_file:
        json.dump(out_data, out_file)

    if res.returncode == 0 and any(
            b.get('error_occurred') for b in in_data.get('benchmarks', [])):
        res.returncode = 1

    return res.returncode


def convert_google_benchark_to_jenkins_benchmark(
    in_data,
    overlay_data=None,
    package_name=None,
):
    group_name = os.path.basename(in_data['context']['executable'])
    if package_name:
        group_name = '%s.%s' % (package_name, group_name)

    out_data = {
        group_name: {},
    }
    for benchmark in in_data.get('benchmarks', []):
        benchmark_type = benchmark['run_type']
        if benchmark_type == 'aggregate':
            out_data[group_name][benchmark['name']] = convert_aggregate_benchmark(benchmark)
        elif benchmark_type == 'iteration':
            out_data[group_name][benchmark['name']] = convert_iteration_benchmark(benchmark)
        else:
            print("WARNING: Unsupported benchmark type '%s'" % benchmark_type, file=sys.stderr)

    if not out_data[group_name]:
        print(
            'WARNING: The performance test results file contained no results',
            file=sys.stderr)

    if overlay_data:
        _merge_results(out_data[group_name], overlay_data.get(group_name, {}))

    return out_data


def convert_aggregate_benchmark(in_data):
    out_data = {
        'parameters': {
            'repetitions': {
                'value': in_data['repetitions'],
            },
        },
    }

    value_override = None
    if in_data.get('error_occurred', False):
        value_override = 'failure'

    out_data.update(convert_extra_metrics(
        in_data, common_aggregate_test_properties, value_override))

    return out_data


def convert_iteration_benchmark(in_data):
    out_data = {
        'parameters': {
            'iterations': {
                'value': in_data['iterations'],
            },
            'repetitions': {
                'value': in_data['repetitions'],
            },
        },
        'cpu_time': {
            'unit': in_data['time_unit'],
        },
        'real_time': {
            'unit': in_data['time_unit'],
        },
    }

    value_override = None
    if in_data.get('error_occurred', False):
        value_override = 'failure'
        out_data['cpu_time']['value'] = value_override
        out_data['real_time']['value'] = value_override
    else:
        out_data['cpu_time']['dblValue'] = in_data['cpu_time']
        out_data['real_time']['dblValue'] = in_data['real_time']

    out_data.update(convert_extra_metrics(
        in_data, common_iteration_test_properties, value_override))

    return out_data


def convert_extra_metrics(in_data, properties_to_ignore, value_override=None):
    for k, v in in_data.items():
        if k in properties_to_ignore:
            continue
        if value_override:
            yield k, {
                'value': value_override,
            }
        elif isinstance(v, bool):
            yield k, {
                'boolValue': 'true' if v else 'false',
            }
        elif isinstance(v, (int, float)):
            yield k, {
                'dblValue': v,
            }
        else:
            yield k, {
                'value': v,
            }


def _merge_results(target, overlay):
    for k, v in overlay.items():
        if isinstance(v, dict) and isinstance(target.get(k), dict):
            _merge_results(target[k], v)
        else:
            target[k] = v



#!/usr/bin/env python3

# Copyright 2014-2015 Open Source Robotics Foundation, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import os
import sys

from ament_package.templates import get_environment_hook_template_path
from ament_package.templates import get_package_level_template_names
from ament_package.templates import get_package_level_template_path
from ament_package.templates import get_prefix_level_template_names
from ament_package.templates import get_prefix_level_template_path

IS_WINDOWS = os.name == 'nt'


def main(argv=sys.argv[1:]):
    """
    Extract the information about templates provided by ament_package.

    Call the API provided by ament_package and
    print CMake code defining several variables containing information about
    the available templates.
    """
    parser = argparse.ArgumentParser(
        description='Extract information about templates provided by '
                    'ament_package and print CMake code defining several '
                    'variables',
    )
    parser.add_argument(
        'outfile',
        nargs='?',
        help='The filename where the output should be written to',
    )
    args = parser.parse_args(argv)

    lines = generate_cmake_code()
    if args.outfile:
        basepath = os.path.dirname(args.outfile)
        if not os.path.exists(basepath):
            os.makedirs(basepath)
        with open(args.outfile, 'w') as f:
            for line in lines:
                f.write('%s\n' % line)
    else:
        for line in lines:
            print(line)


def generate_cmake_code():
    """
    Return a list of CMake set() commands containing the template information.

    :returns: list of str
    """
    variables = []

    if not IS_WINDOWS:
        variables.append((
            'ENVIRONMENT_HOOK_LIBRARY_PATH',
            '"%s"' % get_environment_hook_template_path('library_path.sh')))
    else:
        variables.append(('ENVIRONMENT_HOOK_LIBRARY_PATH', ''))

    ext = '.bat.in' if IS_WINDOWS else '.sh.in'
    variables.append((
        'ENVIRONMENT_HOOK_PYTHONPATH',
        '"%s"' % get_environment_hook_template_path('pythonpath' + ext)))

    templates = []
    for name in get_package_level_template_names():
        templates.append('"%s"' % get_package_level_template_path(name))
    variables.append((
        'PACKAGE_LEVEL',
        templates))

    templates = []
    for name in get_prefix_level_template_names():
        templates.append('"%s"' % get_prefix_level_template_path(name))
    variables.append((
        'PREFIX_LEVEL',
        templates))

    lines = []
    for (k, v) in variables:
        if isinstance(v, list):
            lines.append('set(ament_cmake_package_templates_%s "")' % k)
            for vv in v:
                lines.append('list(APPEND ament_cmake_package_templates_%s %s)'
                             % (k, vv))
        else:
            lines.append('set(ament_cmake_package_templates_%s %s)' % (k, v))
    # Ensure backslashes are replaced with forward slashes because CMake cannot
    # parse files with backslashes in it.
    return [line.replace('\\', '/') for line in lines]


if __name__ == '__main__':
    main()



# Copyright 2023 Open Source Robotics Foundation, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

def test_print_unicode():
    '''
    Print a non UTF-8 byte sequence in a test to verify stderr parsing.
    This value was originally reported in: https://github.com/colcon/colcon-core/issues/468
   '''
    sys.stderr.buffer.write(b'val: ')
    sys.stderr.buffer.write(b'\xff\x00\xa1')
    sys.stderr.buffer.write(b'\n')
